@@ -1,11 +1,10 @@
 #![deny(missing_docs)]
 #![warn(rust_2018_idioms)]
-#![doc(html_root_url = "https://docs.rs/tcp-stream/0.20.10/")]
+#![doc(html_root_url = "https://docs.rs/tcp-stream/0.23.0/")]
 
-//! # mio's TCP stream on steroids
+//! # std::net::TCP stream on steroids
 //!
-//! tcp-stream is a library aiming at providing TLS and futures/tokio
-//! support to mio's TcpStream without forcibly using tokio-reactor
+//! tcp-stream is a library aiming at providing TLS support to std::net::TcpStream
 //!
 //! # Examples
 //!
@@ -18,6 +17,7 @@
 //!
 //! fn main() {
 //!     let mut stream = TcpStream::connect("google.com:443").unwrap();
+//!     stream.set_nonblocking(true).unwrap();
 //!
 //!     while !stream.is_connected() {
 //!         if stream.try_connect().unwrap() {
@@ -50,8 +50,6 @@
 //! ```
 
 use cfg_if::cfg_if;
-use mio::{event::Source, net::TcpStream as MioTcpStream, Interest, Registry, Token};
-
 use std::{
     convert::TryFrom,
     error::Error,
@@ -66,61 +64,61 @@ use std::{
 use native_tls_crate as native_tls;
 
 #[cfg(feature = "native-tls")]
-/// Reexport native-tls's TlsConnector
+/// Reexport native-tls's `TlsConnector`
 pub use native_tls::TlsConnector as NativeTlsConnector;
 
 #[cfg(feature = "native-tls")]
-/// A TcpStream wrapped by native-tls
+/// A `TcpStream` wrapped by native-tls
 pub type NativeTlsStream = native_tls::TlsStream<TcpStream>;
 
 #[cfg(feature = "native-tls")]
-/// A MidHandshakeTlsStream from native-tls
+/// A `MidHandshakeTlsStream` from native-tls
 pub type NativeTlsMidHandshakeTlsStream = native_tls::MidHandshakeTlsStream<TcpStream>;
 
 #[cfg(feature = "native-tls")]
-/// A HandshakeError from native-tls
+/// A `HandshakeError` from native-tls
 pub type NativeTlsHandshakeError = native_tls::HandshakeError<TcpStream>;
 
 #[cfg(feature = "openssl")]
-/// Reexport openssl's TlsConnector
+/// Reexport openssl's `TlsConnector`
 pub use openssl::ssl::{SslConnector as OpenSslConnector, SslMethod as OpenSslMethod};
 
 #[cfg(feature = "openssl")]
-/// A TcpStream wrapped by openssl
+/// A `TcpStream` wrapped by openssl
 pub type OpenSslStream = openssl::ssl::SslStream<TcpStream>;
 
 #[cfg(feature = "openssl")]
-/// A MidHandshakeTlsStream from openssl
+/// A `MidHandshakeTlsStream` from openssl
 pub type OpenSslMidHandshakeTlsStream = openssl::ssl::MidHandshakeSslStream<TcpStream>;
 
 #[cfg(feature = "openssl")]
-/// A HandshakeError from openssl
+/// A `HandshakeError` from openssl
 pub type OpenSslHandshakeError = openssl::ssl::HandshakeError<TcpStream>;
 
 #[cfg(feature = "openssl")]
-/// An ErrorStack from openssl
+/// An `ErrorStack` from openssl
 pub type OpenSslErrorStack = openssl::error::ErrorStack;
 
 #[cfg(feature = "rustls-connector")]
-/// Reexport rustls-connector's TlsConnector
+/// Reexport rustls-connector's `TlsConnector`
 pub use rustls_connector::{RustlsConnector, RustlsConnectorConfig};
 
 #[cfg(feature = "rustls-connector")]
-/// A TcpStream wrapped by rustls
+/// A `TcpStream` wrapped by rustls
 pub type RustlsStream = rustls_connector::TlsStream<TcpStream>;
 
 #[cfg(feature = "rustls-connector")]
-/// A MidHandshakeTlsStream from rustls_connector
+/// A `MidHandshakeTlsStream` from rustls-connector
 pub type RustlsMidHandshakeTlsStream = rustls_connector::MidHandshakeTlsStream<TcpStream>;
 
 #[cfg(feature = "rustls-connector")]
-/// A HandshakeError from rustls_connector
+/// A `HandshakeError` from rustls-connector
 pub type RustlsHandshakeError = rustls_connector::HandshakeError<TcpStream>;
 
 /// Wrapper around plain or TLS TCP streams
 pub enum TcpStream {
-    /// Wrapper around mio's TcpStream
-    Plain(MioTcpStream, bool),
+    /// Wrapper around std::net::TcpStream
+    Plain(StdTcpStream, bool),
     #[cfg(feature = "native-tls")]
     /// Wrapper around a TLS stream hanled by native-tls
     NativeTls(Box<NativeTlsStream>),
@@ -151,7 +149,8 @@ pub struct OwnedTLSConfig {
 }
 
 impl OwnedTLSConfig {
-    /// Get the ephemeral TLSConfig corresponding to the OwnedTLSConfig
+    /// Get the ephemeral `TLSConfig` corresponding to the `OwnedTLSConfig`
+    #[must_use]
     pub fn as_ref(&self) -> TLSConfig<'_, '_, '_> {
         TLSConfig {
             identity: self.identity.as_ref().map(OwnedIdentity::as_ref),
@@ -179,7 +178,8 @@ pub struct OwnedIdentity {
 }
 
 impl OwnedIdentity {
-    /// Get the ephemeral Identity corresponding to the OwnedIdentity
+    /// Get the ephemeral `Identity` corresponding to the `OwnedIdentity`
+    #[must_use]
     pub fn as_ref(&self) -> Identity<'_, '_> {
         Identity {
             der: &self.der,
@@ -188,27 +188,27 @@ impl OwnedIdentity {
     }
 }
 
-/// Holds either the TLS TcpStream result or the current handshake state
+/// Holds either the TLS `TcpStream` result or the current handshake state
 pub type HandshakeResult = Result<TcpStream, HandshakeError>;
 
 impl TcpStream {
-    /// Wrapper around mio's TcpStream::connect inspired by std::net::TcpStream::connect
+    /// Wrapper around `std::net::TcpStream::connect`
     pub fn connect<A: ToSocketAddrs>(addr: A) -> io::Result<Self> {
-        connect_mio(addr, None).and_then(Self::try_from)
+        connect_std(addr, None).and_then(Self::try_from)
     }
 
-    /// Wrapper around mio's TcpStream::connect inspired by std::net::TcpStream::connect_timeout
-    /// and std::net::TcpStream::connect. We used the timeout on the first SocketAddr.
+    /// Wrapper around `std::net::TcpStream::connect_timeout`
     pub fn connect_timeout<A: ToSocketAddrs>(addr: A, timeout: Duration) -> io::Result<Self> {
-        connect_mio(addr, Some(timeout)).and_then(Self::try_from)
+        connect_std(addr, Some(timeout)).and_then(Self::try_from)
     }
 
-    /// Wrapper around mio's TcpStream::from_std
+    /// Convert from a `std::net::TcpStream`
     pub fn from_std(stream: StdTcpStream) -> io::Result<Self> {
-        Self::try_from(MioTcpStream::from_std(stream))
+        Self::try_from(stream)
     }
 
     /// Check whether the stream is connected or not
+    #[must_use]
     pub fn is_connected(&self) -> bool {
         if let Self::Plain(_, connected) = self {
             *connected
@@ -255,7 +255,7 @@ impl TcpStream {
     /// Enable TLS using native-tls
     pub fn into_native_tls(
         self,
-        connector: NativeTlsConnector,
+        connector: &NativeTlsConnector,
         domain: &str,
     ) -> Result<Self, HandshakeError> {
         Ok(connector.connect(domain, self.into_plain()?)?.into())
@@ -265,7 +265,7 @@ impl TcpStream {
     /// Enable TLS using openssl
     pub fn into_openssl(
         self,
-        connector: OpenSslConnector,
+        connector: &OpenSslConnector,
         domain: &str,
     ) -> Result<Self, HandshakeError> {
         Ok(connector.connect(domain, self.into_plain()?)?.into())
@@ -275,7 +275,7 @@ impl TcpStream {
     /// Enable TLS using rustls
     pub fn into_rustls(
         self,
-        connector: RustlsConnector,
+        connector: &RustlsConnector,
         domain: &str,
     ) -> Result<Self, HandshakeError> {
         Ok(connector.connect(domain, self.into_plain()?)?.into())
@@ -294,24 +294,20 @@ impl TcpStream {
     }
 }
 
-fn connect_mio<A: ToSocketAddrs>(addr: A, timeout: Option<Duration>) -> io::Result<MioTcpStream> {
+fn connect_std<A: ToSocketAddrs>(addr: A, timeout: Option<Duration>) -> io::Result<StdTcpStream> {
     let mut addrs = addr.to_socket_addrs()?;
     let mut err = None;
     if let Some(timeout) = timeout {
         if let Some(addr) = addrs.next() {
-            match StdTcpStream::connect_timeout(&addr, timeout)
-                .and_then(|stream| stream.set_nonblocking(true).map(|()| stream))
-            {
-                Ok(stream) => return Ok(MioTcpStream::from_std(stream)),
+            match StdTcpStream::connect_timeout(&addr, timeout) {
+                Ok(stream) => return Ok(stream),
                 Err(error) => err = Some(error),
             }
         }
     }
     for addr in addrs {
-        match StdTcpStream::connect(&addr)
-            .and_then(|stream| stream.set_nonblocking(true).map(|()| stream))
-        {
-            Ok(stream) => return Ok(MioTcpStream::from_std(stream)),
+        match StdTcpStream::connect(addr) {
+            Ok(stream) => return Ok(stream),
             Err(error) => err = Some(error),
         }
     }
@@ -325,18 +321,22 @@ fn into_rustls_common(
     s: TcpStream,
     mut c: RustlsConnectorConfig,
     domain: &str,
-    mut config: TLSConfig<'_, '_, '_>,
+    config: TLSConfig<'_, '_, '_>,
 ) -> HandshakeResult {
     use rustls_connector::rustls::{Certificate, PrivateKey};
 
-    if let Some(identity) = config.identity {
-        let pfx = p12::PFX::parse(identity.der)
-            .map_err(|err| io::Error::new(io::ErrorKind::Other, err))?;
+    if let Some(cert_chain) = config.cert_chain {
+        let mut cert_chain = std::io::BufReader::new(cert_chain.as_bytes());
+        let certs = rustls_pemfile::certs(&mut cert_chain)
+            .map_err(|err| io::Error::new(io::ErrorKind::InvalidData, err))?;
+        c.add_parsable_certificates(&certs);
+    }
+    let connector = if let Some(identity) = config.identity {
+        let pfx = pkcs_12::PFX::parse(identity.der).map_err(io::Error::from)?;
         let key = if let Some(key) = pfx
             .key_bags(identity.password)
-            .map_err(|err| io::Error::new(io::ErrorKind::Other, err))?
-            .iter()
-            .next()
+            .map_err(io::Error::from)?
+            .get(0)
         {
             PrivateKey(key.clone())
         } else {
@@ -346,21 +346,16 @@ fn into_rustls_common(
         };
         let certs = pfx
             .cert_bags(identity.password)
-            .map_err(|err| io::Error::new(io::ErrorKind::Other, err))?
+            .map_err(io::Error::from)?
             .iter()
             .map(|cert| Certificate(cert.clone()))
             .collect();
-        c.set_single_client_cert(certs, key)
-            .map_err(|err| io::Error::new(io::ErrorKind::Other, err))?;
-    }
-    if let Some(cert_chain) = config.cert_chain.as_mut() {
-        c.root_store
-            .add_pem_file(&mut cert_chain.as_bytes())
-            .map_err(|()| {
-                io::Error::new(io::ErrorKind::Other, "Failed to import certificates chain")
-            })?;
-    }
-    s.into_rustls(c.into(), domain)
+        c.connector_with_single_cert(certs, key)
+            .map_err(|err| io::Error::new(io::ErrorKind::Other, err))?
+    } else {
+        c.connector_with_no_client_auth()
+    };
+    s.into_rustls(&connector, domain)
 }
 
 cfg_if! {
@@ -396,7 +391,7 @@ cfg_if! {
                     builder.cert_store_mut().add_cert(cert)?;
                 }
             }
-            s.into_openssl(builder.build(), domain)
+            s.into_openssl(&builder.build(), domain)
         }
     } else if #[cfg(feature = "native-tls")] {
         fn into_tls_impl(s: TcpStream, domain: &str, config: TLSConfig<'_, '_, '_>) -> HandshakeResult {
@@ -406,12 +401,15 @@ cfg_if! {
             if let Some(identity) = config.identity {
                 builder.identity(native_tls::Identity::from_pkcs12(identity.der, identity.password).map_err(|e| io::Error::new(io::ErrorKind::Other, e))?);
             }
-            if let Some(cert_chain) = config.cert_chain.as_ref() {
-                for cert in pem::parse_many(cert_chain).iter().rev() {
-                    builder.add_root_certificate(Certificate::from_der(&cert.contents[..]).map_err(|e| io::Error::new(io::ErrorKind::Other, e))?);
+            if let Some(cert_chain) = config.cert_chain {
+                let mut cert_chain = std::io::BufReader::new(cert_chain.as_bytes());
+                for cert in rustls_pemfile::read_all(&mut cert_chain)?.iter().rev() {
+                    if let rustls_pemfile::Item::X509Certificate(cert) = cert {
+                        builder.add_root_certificate(Certificate::from_der(&cert[..]).map_err(|e| io::Error::new(io::ErrorKind::Other, e))?);
+                    }
                 }
             }
-            s.into_native_tls(builder.build().map_err(|e| io::Error::new(io::ErrorKind::Other, e))?, domain)
+            s.into_native_tls(&builder.build().map_err(|e| io::Error::new(io::ErrorKind::Other, e))?, domain)
         }
     } else {
         fn into_tls_impl(s: TcpStream, _domain: &str, _: TLSConfig<'_, '_, '_>) -> HandshakeResult {
@@ -420,10 +418,10 @@ cfg_if! {
     }
 }
 
-impl TryFrom<MioTcpStream> for TcpStream {
+impl TryFrom<StdTcpStream> for TcpStream {
     type Error = io::Error;
 
-    fn try_from(s: MioTcpStream) -> io::Result<Self> {
+    fn try_from(s: StdTcpStream) -> io::Result<Self> {
         let mut this = TcpStream::Plain(s, false);
         this.try_connect()?;
         Ok(this)
@@ -464,7 +462,7 @@ impl TcpStream {
 }
 
 impl Deref for TcpStream {
-    type Target = MioTcpStream;
+    type Target = StdTcpStream;
 
     fn deref(&self) -> &Self::Target {
         match self {
@@ -551,30 +549,6 @@ impl Write for TcpStream {
     }
 }
 
-impl Source for TcpStream {
-    fn register(
-        &mut self,
-        registry: &Registry,
-        token: Token,
-        interests: Interest,
-    ) -> io::Result<()> {
-        <MioTcpStream as Source>::register(self, registry, token, interests)
-    }
-
-    fn reregister(
-        &mut self,
-        registry: &Registry,
-        token: Token,
-        interests: Interest,
-    ) -> io::Result<()> {
-        <MioTcpStream as Source>::reregister(self, registry, token, interests)
-    }
-
-    fn deregister(&mut self, registry: &Registry) -> io::Result<()> {
-        <MioTcpStream as Source>::deregister(self, registry)
-    }
-}
-
 impl fmt::Debug for TcpStream {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("TcpStream")
@@ -602,6 +576,7 @@ pub enum MidHandshakeTlsStream {
 
 impl MidHandshakeTlsStream {
     /// Get a reference to the inner stream
+    #[must_use]
     pub fn get_ref(&self) -> &TcpStream {
         match self {
             MidHandshakeTlsStream::Plain(mid) => mid,
@@ -615,6 +590,7 @@ impl MidHandshakeTlsStream {
     }
 
     /// Get a mutable reference to the inner stream
+    #[must_use]
     pub fn get_mut(&mut self) -> &mut TcpStream {
         match self {
             MidHandshakeTlsStream::Plain(mid) => mid,
@@ -785,24 +761,26 @@ impl From<io::Error> for HandshakeError {
 #[cfg(unix)]
 mod sys {
     use crate::TcpStream;
-    use mio::net::TcpStream as MioTcpStream;
-    use std::os::unix::io::{AsRawFd, FromRawFd, RawFd};
+    use std::{
+        net::TcpStream as StdTcpStream,
+        os::unix::io::{AsRawFd, FromRawFd, RawFd},
+    };
 
     impl AsRawFd for TcpStream {
         fn as_raw_fd(&self) -> RawFd {
-            <MioTcpStream as AsRawFd>::as_raw_fd(self)
+            <StdTcpStream as AsRawFd>::as_raw_fd(self)
         }
     }
 
     impl AsRawFd for &TcpStream {
         fn as_raw_fd(&self) -> RawFd {
-            <MioTcpStream as AsRawFd>::as_raw_fd(self)
+            <StdTcpStream as AsRawFd>::as_raw_fd(self)
         }
     }
 
     impl FromRawFd for TcpStream {
         unsafe fn from_raw_fd(fd: RawFd) -> Self {
-            Self::Plain(MioTcpStream::from_raw_fd(fd), false)
+            Self::Plain(StdTcpStream::from_raw_fd(fd), false)
         }
     }
 }
@@ -810,24 +788,26 @@ mod sys {
 #[cfg(windows)]
 mod sys {
     use crate::TcpStream;
-    use mio::net::TcpStream as MioTcpStream;
-    use std::os::windows::io::{AsRawSocket, FromRawSocket, RawSocket};
+    use std::{
+        net::TcpStream as StdTcpStream,
+        os::windows::io::{AsRawSocket, FromRawSocket, RawSocket},
+    };
 
     impl AsRawSocket for TcpStream {
         fn as_raw_socket(&self) -> RawSocket {
-            <MioTcpStream as AsRawSocket>::as_raw_socket(self)
+            <StdTcpStream as AsRawSocket>::as_raw_socket(self)
         }
     }
 
     impl AsRawSocket for &TcpStream {
         fn as_raw_socket(&self) -> RawSocket {
-            <MioTcpStream as AsRawSocket>::as_raw_socket(self)
+            <StdTcpStream as AsRawSocket>::as_raw_socket(self)
         }
     }
 
     impl FromRawSocket for TcpStream {
         unsafe fn from_raw_socket(socket: RawSocket) -> Self {
-            Self::Plain(MioTcpStream::from_raw_socket(socket), false)
+            Self::Plain(StdTcpStream::from_raw_socket(socket), false)
         }
     }
 }
